/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { routesWithoutCors } from "../config/routesWithoutCors.js";
import type { onRequestHookHandler } from "fastify";

const isValidOrigin = (origin: string | undefined): boolean => {
  if (origin === undefined) {
    // We do not allow server-to-server requests.
    return false;
  }
  if (process.env.NODE_ENV === "dev" && origin === "http://localhost:4200") {
    // We allow the client to access the server when both are running locally.
    return true;
  }
  // Otherwise, we only allow requests from our web application.
  return origin === "https://hikari.nhcarrigan.com";
};

/**
 * Ensures that form submissions only come from our web application.
 * @param request - The request payload from the server.
 * @param response - The reply handler from Fastify.
 * @returns A Fastify reply if the request is invalid, otherwise undefined.
 */
// eslint-disable-next-line @typescript-eslint/no-misused-promises -- For reasons I cannot comprehend, Fastify seems to require us to return a request?
export const corsHook: onRequestHookHandler = async(request, response) => {
  if (routesWithoutCors.includes(request.url)) {
    return undefined;
  }
  const { origin } = request.headers;
  const { host } = request.headers;
  const { referer } = request.headers;

  const isSameOriginSafe
  = origin === undefined
  && host === "hikari.nhcarrigan.com"
  && referer?.startsWith("https://hikari.nhcarrigan.com");

  if (!isValidOrigin(origin) && isSameOriginSafe !== true) {
    return await response.status(403).send({
      error:

        `This route is only accessible from our dashboard at https://hikari.nhcarrigan.com - your origin of ${String(request.headers.origin)} is invalid..`,
    });
  }
  return undefined;
};
